@@ -1,4 +1,4 @@
-# $Id: Subs.pm,v 1.8 2005/03/28 18:48:56 chip Exp $
+# $Id: Subs.pm,v 1.13 2005/04/06 20:44:03 chip Exp $
 
 package Perl6::Subs;
 
@@ -8,11 +8,11 @@ Perl6::Subs - Define your subroutines in the Perl 6 style
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use strict;
 use warnings;
@@ -31,7 +31,7 @@ use Carp ();
 # Debugging
 #
 
-our $DEBUG ||= $ENV{PERL6_SUBS_DEBUG};
+our $DEBUG ||= $ENV{PERL6_SUBS_DEBUG} || 0;
 
 
 #----------------------------------------------------------------
@@ -540,30 +540,39 @@ my $CS_has_trait =
 use Filter::Simple;
 
 FILTER_ONLY
-  code =>
+  executable =>
     sub {
 	# FIXME: Use This
 	my $ph = $Filter::Simple::placeholder;
 
 	my $code = $_;
       SUB:
-	while ($code =~ m{ \b
+	while ($code =~ m{ # things to skip...
+			   (?:
+			       ^ [ \t]* \# .* $		# comments, we ass_u_me
+			   )
+
+			   |
+
+			   # things not to skip
+			   \b
 			   (
-			     ( sub | ( method ) ) \b	# $3 is 'method' foo
+			     (?: sub | ( method ) ) \b	# $2 is 'method' foo
 			     (?: \s+ ( $word_rx ) )?
 			     \s*
 			   )
-			   (?(3)  (?= \( | \{ | : | is \b | returns \b )
+			   (?(2)  (?= \( | \{ | : | is \b | returns \b )
 				| (?= \( [^\)]* \w ) )
-			 }xog)
+			 }xmog)
 	{
-	    my ($name) = $3;
+	    next SUB unless $1;
 
 	    my $proto_pos = pos($code);
 	    my $sub_pos = $proto_pos - length($1);
+	    my $name = $3;
 
 	    print "Perl6::Subs - trying sub decl at {{" . substr($code, $sub_pos, 40) . "}}\n"
-	      if $DEBUG;
+	      if $DEBUG > 1;
 
 	    my ($sub, $subtext) = Perl6::Subs::Sub->new_from_decl(substr($code, $sub_pos));
 
@@ -614,7 +623,7 @@ FILTER_ONLY
 	    }
 
 	    print "Perl6::Subs - resuming search for sub decls at {{" . substr($code, pos($code), 40) . "}}\n"
-	      if $DEBUG;
+	      if $DEBUG > 1;
 	}
 
 	$_ = $code;
@@ -634,75 +643,108 @@ __END__
 
   use Perl6::Subs;
 
-  sub get (Array $x) { pop @$x }
+  sub foo ($x)                     # Positional parameters
+    { bar($x) }
 
-  method foo { $self->bar }        # default invocant: '$self'
+  sub get (Array $x)               # Type validation
+    { pop @$x }
+
+  sub get_or_die ($x of Array where { @$_ })  # Subtyping
+    { pop @$x }
+
+  sub show (Str $s, IO ?$io)       # Optional parameters
+    { print { $io || *STDOUT } $s }
+
+  sub limit (Int $i, Int +$hi, Int +$low)  # Named parameters
+    { ... }
+
+  method foo                       # Invocant: '$self'
+    { $self->bar }
+
+  method foo (Foo $self: Bar $x)   # Parameter 'isa' classname
+    { $self->use_bar($x) }
+
+And there's more...
 
 =head1 DESCRIPTION
 
-Perl6::Subs lets you write subroutine definitions in Perl6 syntax,
-with some limitations.  Given a subroutine defined with a Perl 6
-prototype, Perl6::Subs generates Perl 5 code for that subroutine that
-will, at runtime: declare the formal parameters, assign them their
-values, and validate their contents.
+Perl6::Subs is a source filter that adds a very useful subset of Perl
+6 subroutine syntax to Perl 5.  Given a subroutine defined with a Perl
+6 prototype, the code generated by Perl6::Subs will, at runtime,
+declare the formal parameters, assign them their values, and validate
+their contents according to both built-in and user-given rules.
 
-Perl6::Subs supports parameters of five categories:
+Perl6::Subs supports all five categories of Perl 6 subroutine
+parameters, here listed in their mandatory order.  (You may skip
+categories, but not reorder them.)
 
 =over
 
 =item *
 
-invocant (e.g. C<$self>) -- marked by a trailing colon rather than the usual comma
+Method invocant, e.g. C<$self:>).  Invocant declarations are marked by
+a trailing colon rather than the usual comma.  To declare an invocant
+when there are no other parameters, just end the prototype with the
+colon.
 
 =item *
 
-mandatory positional -- without decoration
+Mandatory positional, e.g. C<$foo>.
 
 =item *
 
-optional positional -- with a "?" before the variable name
+Optional positional, e.g. C<?$foo>.
 
 =item *
 
-optional named -- with a "+" before the variable name
+Optional named, e.g. C<+$foo>.
 
 =item *
 
-required named -- with a "+" prefix and an C<is required> trait
+Required named, e.g. C<+$foo is required>.
 
 =item *
 
-"slurpy" -- i.e. an array or hash that consumes the rest of the actual
-parameters -- with a "*" before the variable name
+Slurpy, e.g. C<*@rest> or C<*%rest>.  Slurpy parameters must appear
+last, as they consume all remaining parameter values.
 
 =back
 
-Note that any given subroutine can either optional positional parameters
-I<or> named parameters I<or> a slurpy parameter.  As a special exception,
-named parameters may be followed by a slurpy hash; the hash will contain
-all the key/value pairs other than the ones used for named parameters.
-Other combinations are not legal due to limitations of Perl 5 parameter
-conventions.
+Perl 5's limited function call semantics (the single array @_) prevent
+Perl6::Subs from supporting all the features of Perl 6 parameter
+lists.  A given subroutine can have either optional positional
+parameters I<or> named parameters I<or> a slurpy parameter; combining
+them is illegal.  As the lone exception, a subroutine with named
+parameters may also have a slurpy hash; the hash will contain all the
+key/value pairs not explicitly given as named parameters.
 
-=head1 SUPPORTED TYPES
+=head1 TYPES AND VALIDATION
 
-Perl6::Subs knows how to validate the following Perl 6 I<fundamental types>:
+Perl6::Subs understands the following type names, and will validate
+that a declared parameter actually has the given type.  The validation
+code will throw an exception with C<Carp::croak> if validation fails.
+
+First, the fundamental Perl 6 types, which in Perl 6 will be unboxed
+(non-objects):
 
 =over
 
 =item B<str>:
 
-Any defined scalar.  (Note: There is no test for not being a
-reference; given Perl 5's overloading facility, a reference may
-actually have a useful string value if you go ahead and use it.)
+Any defined scalar.  (Note: References are permitted.  Given Perl 5's
+overloading facility, a reference may actually have a useful string
+value if you go ahead and use it.)  Due to the semantics of Perl 6's
+C<str> type, use of C<str> is recommended only for bare byte buffers
+without string semantics.
 
 =item B<num>:
 
-A defined scalar that passes &Scalar::Util::looks_like_number.
+A number; specifically a defined scalar that passes the test of
+C<Scalar::Util::looks_like_number>.
 
 =item B<int>:
 
-A B<num> with no fractional part.
+An integer; specifically, a B<num> with no fractional part.
 
 =item B<ref>:
 
@@ -714,9 +756,9 @@ Any defined scalar.
 
 =back
 
-Perl6::Subs knows how to validate the following Perl 6 C<object types>
-(but note that while Perl 6 considers all these to be objects, Perl 5
-generally does not):
+Now, the C<object types>.  Note that while Perl 6 considers all these
+to be objects, Perl 5 often doesn't.  Also note that, in general,
+C<undef> is permitted as a valid value for all object types.
 
 =over
 
@@ -727,7 +769,7 @@ Any value, including undef.
 =item B<Str>:
 
 Any scalar value, including undef.
-(For Perl6::Subs, this is a synonym for B<Any>.)
+(In Perl 5, this is a synonym for B<Any>.)
 
 =item B<Num>:
 
@@ -759,9 +801,17 @@ A regexp reference (qr//), or undef.
 
 =item B<IO>:
 
-An IO handle (e.g. C<*STDOUT{IO}>, or a glob (e.g. C<*STDOUT>), or a
-glob reference (e.g. C<\*STDOUT>), or undef.  Note that autovivified
-file handles work here, as they are references to blessed globs.
+A value that can be used for I/O: An IO handle reference
+(e.g. C<*STDOUT{IO}>, a glob (e.g. C<*STDOUT>), a glob reference
+(e.g. C<\*STDOUT>), or undef.  Note that autovivified file handles
+are legal C<IO> values, as they are references to (blessed) globs.
+
+=back
+
+Perl6::Subs also supports these type names which are not legal in Perl
+6, but which may be useful in writing Perl 5:
+
+=over
 
 =item B<Glob>:
 
@@ -773,9 +823,99 @@ A glob reference (e.g. C<\*STDOUT>), or undef.
 
 =back
 
-Any other bareword used as a type is assumed to name a user-defined class.
-A parameter so typed must satisfy UNIVERSAL::isa() of that class, or be
-undef.
+Finally, any bareword other than the above used as a type name is
+assumed to name I<a user-defined class>.  A parameter so typed must
+satisfy UNIVERSAL::isa() of that class, or be undef.  If Perl warnings
+are enabled at compile time, a mispelled class name will generate a
+diagnostic.
+
+=head1 SUBROUTINE DECLARATION SYNTAX
+
+Perl6::Subs filters subroutines declared either with a Perl 6 style
+prototype; or with the "method" keyword replacing the word "sub" (in
+which case the "method" trait is implied).  Perl6::Subs also
+understand both Perl 5 and Perl 6 syntax for subroutine traits (Perl 5
+calls them "attributes").  Thus, these declarations are synonymous:
+
+    sub foo is method {...}
+    sub foo returns(Any) is method {...}
+    sub foo ($self:) : method {...}
+    method foo {...}
+    method foo ($self:) {...}
+
+However, this declaration uses no Perl 6 features, and therefore
+Perl6::Subs does not filter it:
+
+    sub foo : method {...}
+
+Perl6::Subs understands subroutine traits after the prototype declared
+with three syntax flavors:
+
+=over
+
+=item *
+
+Perl 6 standard trait syntax, e.g. C<is trait> and C<is trait(params)>
+
+=item *
+
+Perl 5 subroutine attributes, introduced with a colon, e.g. C<:method>
+
+=item *
+
+As a special case, the Perl 6 trait C<returns(TYPE)>, which requires
+neither I<is> nor a colon.
+
+=back
+
+=head1 PARAMETER DECLARATION SYNTAX, WITH A SIDE OF SUBTYPING
+
+Perl6::Subs supports a subset of the Perl 6 type declaration syntax.
+Allowed parameter forms are C<Type $var> and C<$var of Type>.  Thus,
+
+    sub foo (Int $i)
+
+and
+
+    sub foo ($i of Int)
+
+are synonymous.  Any parameter traits must be specified at the end of
+the given declaration.  Thus a required named parameter of type Int
+may be specified as:
+
+    sub foo (Int +$i is required)
+
+or
+
+    sub foo (+$i of Int is required)
+
+(The only implemented parameter trait as of this writing is C<is
+required>, which is only meaningful on named parameters.)
+
+You may create an anonymous I<subtype> (restricted type) using a
+C<where> clause, which specifies a block of code that must evaluate to
+true when C<$_> is set to a given value.  For example, to accept only
+positive integers as parameters, you could write:
+
+    sub foo (Int where { $_ > 0 } $i)
+
+or
+
+    sub foo ($i of Int where { $_ > 0 })
+
+(The latter is clearer, in the author's opinion, since the variable is
+textually closer to the base type.)
+
+=head1 DEBUGGING
+
+If the environment variable C<PERL6_SUBS_DEBUG> is set to a true
+value, Perl6::Subs will print to standard output the complete filtered
+text of any source file in which it is used.
+
+If you're debugging Perl6::Subs itself, the environment variables
+C<PERL6_SUBS_RD_TRACE> and C<PERL6_SUBS_RD_HINTS> set the
+C<$::RD_TRACE> and C<$::RD_HINTS> variables, respectively, opening a
+window on the operation of C<Parse::RecDescent>.
 
 =head1 CAVEATS
 
@@ -858,8 +998,13 @@ expression provided so kindly by Damian.
 
 =item *
 
-The default invocant, currently C<$self>, should be user-changable.  But
-I'm not sure the best user interface.  This would work:
+The default invocant for methods is C<$self>, because that's the
+convention used by about 90% of object-oriented code on CPAN.  (The
+actual default invocant for Perl 6 is somewhat up in the air.)
+
+The default invocant for Perl6::Subs should probably be
+user-changable.  But I'm not sure the best user interface.
+This would work:
 
   use Perl6::Subs invocant => '$this';
 
